﻿; =======================================================================================
; Name ..........: Custom Gui Example
; Description ...: AutoHotkey Gui styled similar to Windows 10 interface
; AHK Version ...: AHK_L 1.1.22.06 (Unicode 32-bit) - September 13, 2015
; Platform ......: Windows 2000+
; Language ......: English (en-US)
; Author ........: Weston Campbell <westoncampbell@gmail.com>
; =======================================================================================

; Global ================================================================================
#SingleInstance, Force ; Allow only one running instance of script
#Persistent ; Keep script permanently running until terminated
#NoEnv ; Avoid checking empty variables to see if they are environment variables
#Warn ; Enable warnings to assist with detecting common errors
;#NoTrayIcon ; Disable the tray icon of the script
SendMode, Input ; Recommended for new scripts due to its superior speed and reliability
SetWorkingDir, %A_ScriptDir% ; Change the working directory of the script
SetBatchLines, -1 ; Run script at maximum speed
SetWinDelay, -1 ; Run window commands at maximum speed
SetControlDelay, -1 ; Run control-modifying commands at maximum speed
OnExit, ExitSub ; Run a subroutine or function automatically when the script exits

ApplicationName := "Application"
; =======================================================================================

; Tray Menu =============================================================================
Menu, Tray, Icon, resources\app-icon.ico ; Replace the default tray icon
Menu, Tray, Tip, % ApplicationName ; Replace the tooltip of the tray icon
Menu, Tray, NoStandard ; Remove all standard tray menu items
Menu, Tray, Add, Reload, ReloadSub
Menu, Tray, Add, ; Separator
Menu, Tray, Add, Exit, ExitSub
; =======================================================================================

; Graphic User Interface ================================================================
Menu, WindowMenu, Add, Refresh, MenuHandler
Menu, WindowMenu, Add,
Menu, WindowMenu, Add, Settings, SettingsShow

GuiW := 600, GuiH := 400

Gui, +LastFound +Resize -Caption -Border -MaximizeBox +MinSize%GuiW%x%GuiH%
hGui1 := WinExist()
Gui, Color, FFFFFF
Gui, Margin, 10, 10

; Window Borders
Gui, Add, Picture, % " x" 0 " y" 0 " w" GuiW " h" 1 " HWNDhWinBorderTop vWinBorderTop", resources\border.png
Gui, Add, Picture, % " x" 0 " y" GuiH-1 " w" GuiW " h" 1 " HWNDhWinBorderBottom vWinBorderBottom", resources\border.png
Gui, Add, Picture, % " x" 0 " y" 1 " w" 1 " h" GuiH-2 " HWNDhWinBorderLeft vWinBorderLeft", resources\border.png
Gui, Add, Picture, % " x" GuiW-1 " y" 1 " w" 1 " h" GuiH-2 " HWNDhWinBorderRight vWinBorderRight", resources\border.png

; Window Menu Button
Gui, Add, Picture, % " x" 1 " y" 1 " w" 46 " h" 31 " Hidden0 HWNDhBtnMenuN vBtnMenuN", resources\button-menu-normal.png
Gui, Add, Picture, % " x" 1 " y" 1 " w" 46 " h" 31 " Hidden1 HWNDhBtnMenuH vBtnMenuH", resources\button-menu-hover.png
Gui, Add, Picture, % " x" 1 " y" 1 " w" 46 " h" 31 " Hidden1 HWNDhBtnMenuP vBtnMenuP", resources\button-menu-pressed.png

; Window Back Button
Gui, Add, Picture, % " x" 1 " y" 1 " w" 46 " h" 31 " Hidden1 HWNDhBtnBackN vBtnBackN", resources\button-back-normal.png
Gui, Add, Picture, % " x" 1 " y" 1 " w" 46 " h" 31 " Hidden1 HWNDhBtnBackH vBtnBackH", resources\button-back-hover.png
Gui, Add, Picture, % " x" 1 " y" 1 " w" 46 " h" 31 " Hidden1 HWNDhBtnBackP vBtnBackP", resources\button-back-pressed.png

; Window Minimize Button
Gui, Add, Picture, % " x" GuiW-139 " y" 1 " w" 46 " h" 31 " Hidden0 HWNDhBtnMinimizeN vBtnMinimizeN", resources\button-minimize-normal.png
Gui, Add, Picture, % " x" GuiW-139 " y" 1 " w" 46 " h" 31 " Hidden1 HWNDhBtnMinimizeH vBtnMinimizeH", resources\button-minimize-hover.png
Gui, Add, Picture, % " x" GuiW-139 " y" 1 " w" 46 " h" 31 " Hidden1 HWNDhBtnMinimizeP vBtnMinimizeP", resources\button-minimize-pressed.png

; Window Maximize Button
Gui, Add, Picture, % " x" GuiW-93 " y" 1 " w" 46 " h" 31 " Hidden0 HWNDhBtnMaximizeN vBtnMaximizeN", resources\button-maximize-normal.png
Gui, Add, Picture, % " x" GuiW-93 " y" 1 " w" 46 " h" 31 " Hidden1 HWNDhBtnMaximizeH vBtnMaximizeH", resources\button-maximize-hover.png
Gui, Add, Picture, % " x" GuiW-93 " y" 1 " w" 46 " h" 31 " Hidden1 HWNDhBtnMaximizeP vBtnMaximizeP", resources\button-maximize-pressed.png

; Window Restore Button
Gui, Add, Picture, % " x" GuiW-93 " y" 1 " w" 46 " h" 31 " Hidden1 HWNDhBtnRestoreN vBtnRestoreN", resources\button-restore-normal.png
Gui, Add, Picture, % " x" GuiW-93 " y" 1 " w" 46 " h" 31 " Hidden1 HWNDhBtnRestoreH vBtnRestoreH", resources\button-restore-hover.png
Gui, Add, Picture, % " x" GuiW-93 " y" 1 " w" 46 " h" 31 " Hidden1 HWNDhBtnRestoreP vBtnRestoreP", resources\button-restore-pressed.png

; Window Close Button
Gui, Add, Picture, % " x" GuiW-47 " y" 1 " w" 46 " h" 31 " Hidden0 HWNDhBtnCloseN vBtnCloseN", resources\button-close-normal.png
Gui, Add, Picture, % " x" GuiW-47 " y" 1 " w" 46 " h" 31 " Hidden1 HWNDhBtnCloseH vBtnCloseH", resources\button-close-hover.png
Gui, Add, Picture, % " x" GuiW-47 " y" 1 " w" 46 " h" 31 " Hidden1 HWNDhBtnCloseP vBtnCloseP", resources\button-close-pressed.png

Gui, Add, Picture, % " x" 1 " y" 32 " w" GuiW-2 " h" GuiH-33 " Hidden1 HWNDhBgSettings vBgSettings", resources\bg-settings.png

Gui, Font, cFFFFFF s20, Segoe UI ; Set font options
Gui, Add, Text, % " x" 38 " y" 60 " w" GuiW-80 " Hidden1 HWNDhSettingsTitle vSettingsTitle +BackgroundTrans", Settings
Gui, Font ; Reset font options

Gui, Font, cFFFFFF s12, Segoe UI ; Set font options
Gui, Add, Text, % " x" 38 " y" 114 " w" GuiW-80 " Hidden1 HWNDhSettingsOption1 vSettingsOption1 +BackgroundTrans", Option 1
Gui, Add, Text, % " x" 38 " y" 164 " w" GuiW-80 " Hidden1 HWNDhSettingsOption2 vSettingsOption2 +BackgroundTrans", Option 2
Gui, Add, Text, % " x" 38 " y" 214 " w" GuiW-80 " Hidden1 HWNDhSettingsOption3 vSettingsOption3 +BackgroundTrans", Option 3
Gui, Font ; Reset font options
Gui, Font, c000000 s9, Segoe UI ; Set font options

Gui, Add, Text, % " x" 59 " y" 8 " w" 200 " +BackgroundTrans", % ApplicationName ; Window Title Text
Gui, Font ; Reset font options

Gui, Show, % " w" GuiW " h" GuiH, % ApplicationName ; Show main window

OnMessage(0x200,"WM_MOUSEMOVE")
OnMessage(0x201,"WM_LBUTTONDOWN")
OnMessage(0x202,"WM_LBUTTONUP")
OnMessage(0x2A3,"WM_MOUSELEAVE")
OnMessage(0x84, "WM_NCHITTEST")
OnMessage(0x83, "WM_NCCALCSIZE")
OnMessage(0x86, "WM_NCACTIVATE")

VarSetCapacity(TME,16,0), NumPut(16,TME,0), NumPut(2,TME,4), NumPut(hGui1,TME,8)

Return ; End automatic execution
; =======================================================================================

; Labels ================================================================================
MaximizeWindow:
	WinGetPos, WinX, WinY, WinW, WinH, A
	GuiControl, Hide, BtnMaximizeN
	GuiControl, Show, BtnRestoreN
	SysGet, MonitorWorkArea, MonitorWorkArea
    WinMove, % ApplicationName,, 0, 0, MonitorWorkAreaRight - MonitorWorkAreaLeft, MonitorWorkAreaBottom - MonitorWorkAreaTop
	Gui, -Resize
Return

RestoreWindow:
	GuiControl, Hide, BtnRestoreN
	GuiControl, Show, BtnMaximizeN
	WinMove, % ApplicationName,, WinX, WinY, WinW, WinH
	Gui, +Resize
Return

SettingsShow:
	GuiControl, Hide, BtnMenuN
	GuiControl, Show, BtnBackN
	GuiControl, Show, BgSettings
	GuiControl, Show, SettingsTitle
	GuiControl, Show, SettingsOption1
	GuiControl, Show, SettingsOption2
	GuiControl, Show, SettingsOption3
Return

SettingsHide:
	GuiControl, Hide, BtnBackN
	GuiControl, Show, BtnMenuN
	GuiControl, Hide, BgSettings
	GuiControl, Hide, SettingsTitle
	GuiControl, Hide, SettingsOption1
	GuiControl, Hide, SettingsOption2
	GuiControl, Hide, SettingsOption3
Return

ReloadSub:
	Reload
Return

MenuHandler:
	MsgBox,, %A_ThisMenuItem%, You selected %A_ThisMenuItem% from menu %A_ThisMenu%.
Return

GuiEscape:
GuiClose:
ExitSub:
	ExitApp ; Terminate the script unconditionally
Return

GuiSize:
	If (ErrorLevel = 1) { ; The window has been minimized
		Return
	} Else If (ErrorLevel = 2) { ; The window has been maximized
		Return
	} Else If (ErrorLevel = 0) { ; The window has been restored, or resized normally
		GuiControl, Move, WinBorderTop, % " x" 0 " y" 0 " w" A_GuiWidth " h" 1
		GuiControl, Move, WinBorderBottom, % " x" 0 " y" A_GuiHeight-1 " w" A_GuiWidth " h" 1
		GuiControl, Move, WinBorderLeft, % " x" 0 " y" 1 " w" 1 " h" A_GuiHeight-2
		GuiControl, Move, WinBorderRight, % " x" A_GuiWidth-1 " y" 1 " w" 1 " h" A_GuiHeight-2
		GuiControl, Move, BtnMinimizeN, % " x" A_GuiWidth-139 " y" 1 " w" 46 " h" 31
		GuiControl, Move, BtnMinimizeH, % " x" A_GuiWidth-139 " y" 1 " w" 46 " h" 31
		GuiControl, Move, BtnMinimizeP, % " x" A_GuiWidth-139 " y" 1 " w" 46 " h" 31
		GuiControl, Move, BtnMaximizeN, % " x" A_GuiWidth-93 " y" 1 " w" 46 " h" 31
		GuiControl, Move, BtnMaximizeH, % " x" A_GuiWidth-93 " y" 1 " w" 46 " h" 31
		GuiControl, Move, BtnMaximizeP, % " x" A_GuiWidth-93 " y" 1 " w" 46 " h" 31
		GuiControl, Move, BtnRestoreN, % " x" A_GuiWidth-93 " y" 1 " w" 46 " h" 31
		GuiControl, Move, BtnRestoreH, % " x" A_GuiWidth-93 " y" 1 " w" 46 " h" 31
		GuiControl, Move, BtnRestoreP, % " x" A_GuiWidth-93 " y" 1 " w" 46 " h" 31
		GuiControl, Move, BtnCloseN, % " x" A_GuiWidth-47 " y" 1 " w" 46 " h" 31
		GuiControl, Move, BtnCloseH, % " x" A_GuiWidth-47 " y" 1 " w" 46 " h" 31
		GuiControl, Move, BtnCloseP, % " x" A_GuiWidth-47 " y" 1 " w" 46 " h" 31
		GuiControl, Move, BGSettings, % " x" 1 " y" 32 " w" A_GuiWidth-2 " h" A_GuiHeight-33
		GuiControl, Move, TreeView1, % " x" 11 " y" 42 " w" 180 " h" A_GuiHeight-53
		Winset, Redraw ; Redraw the controls in case of error in appearance
	}
Return
; =======================================================================================

; Functions =============================================================================
WM_MOUSEMOVE(wParam,lParam,msg,hwnd) {
	Global

	DllCall("TrackMouseEvent","uint",&TME)

	MouseGetPos,,,, MouseCtrl, 2 ; Get HWND of control under mouse cursor

	If (MouseCtrl = hBtnMenuN || MouseCtrl = hBtnMenuH)	{
		GuiControl, Show, BtnMenuH
	} Else {
		GuiControl, Hide, BtnMenuH
	}
	If (MouseCtrl = hBtnBackN || MouseCtrl = hBtnBackH)	{
		GuiControl, Show, BtnBackH
	} Else {
		GuiControl, Hide, BtnBackH
	}
	If (MouseCtrl = hBtnMinimizeN || MouseCtrl = hBtnMinimizeH)	{
		GuiControl, Show, BtnMinimizeH
	} Else {
		GuiControl, Hide, BtnMinimizeH
	}
	If (MouseCtrl = hBtnMaximizeN || MouseCtrl = hBtnMaximizeH)	{
		GuiControl, Show, BtnMaximizeH
	} Else {
		GuiControl, Hide, BtnMaximizeH
	}
	If (MouseCtrl = hBtnRestoreN || MouseCtrl = hBtnRestoreH)	{
		GuiControl, Show, BtnRestoreH
	} Else {
		GuiControl, Hide, BtnRestoreH
	}
	If (MouseCtrl = hBtnCloseN || MouseCtrl = hBtnCloseH) {
		GuiControl, Show, BtnCloseH
	} Else {
		GuiControl, Hide, BtnCloseH
	}
}

WM_LBUTTONDOWN(wParam,lParam,msg,hwnd) {
	Global

	If (MouseCtrl = hBtnMenuN || MouseCtrl = hBtnMenuH) {
		GuiControl, Show, BtnMenuP
	} Else If (MouseCtrl = hBtnBackN || MouseCtrl = hBtnBackH) {
		GuiControl, Show, BtnBackP
	} Else If (MouseCtrl = hBtnMinimizeN || MouseCtrl = hBtnMinimizeH) {
		GuiControl, Show, BtnMinimizeP
	} Else If (MouseCtrl = hBtnMaximizeN || MouseCtrl = hBtnMaximizeH) {
		GuiControl, Show, BtnMaximizeP
	} Else If (MouseCtrl = hBtnRestoreN || MouseCtrl = hBtnRestoreH) {
		GuiControl, Show, BtnRestoreP
	} Else If (MouseCtrl = hBtnCloseN || MouseCtrl = hBtnCloseH) {
		GuiControl, Show, BtnCloseP
	} Else If (A_Gui = 1) {
		WinGet, Style, Style, A
		If (Style & 0x40000) { ; Disable dragging if window isn't resizable
			PostMessage, 0xA1, 2 ; WM_NCLBUTTONDOWN
		}
	}
}

WM_LBUTTONUP(wParam,lParam,msg,hwnd) {
	Global

	If (MouseCtrl = hBtnMenuN || MouseCtrl = hBtnMenuH || MouseCtrl = hBtnMenuP) {
		Menu, WindowMenu, Show
	} Else If (MouseCtrl = hBtnBackN || MouseCtrl = hBtnBackH || MouseCtrl = hBtnBackP) {
		GoSub, SettingsHide
	} Else If (MouseCtrl = hBtnMinimizeN || MouseCtrl = hBtnMinimizeH || MouseCtrl = hBtnMinimizeP) {
		WinMinimize
	} Else If (MouseCtrl = hBtnMaximizeN || MouseCtrl = hBtnMaximizeH || MouseCtrl = hBtnMaximizeP) {
		GoSub, MaximizeWindow
	} Else If (MouseCtrl = hBtnRestoreN || MouseCtrl = hBtnRestoreH || MouseCtrl = hBtnRestoreP) {
		GoSub, RestoreWindow
	} Else If (MouseCtrl = hBtnCloseN || MouseCtrl = hBtnCloseH || MouseCtrl = hBtnCloseP) {
		GoSub, ExitSub
	}

	GuiControl, Hide, BtnMenuP
	GuiControl, Hide, BtnBackP
	GuiControl, Hide, BtnMinimizeP
	GuiControl, Hide, BtnMaximizeP
	GuiControl, Hide, BtnRestoreP
	GuiControl, Hide, BtnCloseP
}

WM_MOUSELEAVE(wParam,lParam,msg,hwnd) {
	Global

	GuiControl, Hide, BtnMenuH
	GuiControl, Hide, BtnMenuP
	GuiControl, Hide, BtnBackH
	GuiControl, Hide, BtnBackP
	GuiControl, Hide, BtnMinimizeH
	GuiControl, Hide, BtnMinimizeP
	GuiControl, Hide, BtnMaximizeH
	GuiControl, Hide, BtnMaximizeP
	GuiControl, Hide, BtnRestoreH
	GuiControl, Hide, BtnRestoreP
	GuiControl, Hide, BtnCloseH
	GuiControl, Hide, BtnCloseP
}

WM_NCCALCSIZE() { ; Size the client area to fill the entire window
	If (A_Gui = 1) {
		Return 0
	}
}

WM_NCACTIVATE() { ; Prevents a border from being drawn when the window is activated
	If (A_Gui = 1) {
		Return 1
	}
}

WM_NCHITTEST(wParam, lParam) { ; Redefine where the sizing borders are
	Static Border_Size = 6

	If (A_Gui != 1)	{
		Return
	}

	WinGetPos, gX, gY, gW, gH

	x := lParam<<48>>48
	y := lParam<<32>>48

	Hit_Left := x<gX+Border_Size
	Hit_Right := x>=gX+gW-Border_Size
	Hit_Top := y<gY+Border_Size
	Hit_Bottom := y>=gY+gH-Border_Size

	If (Hit_Top = 1) {
		If (Hit_Left = 1) {
			Return 0xD
		}
		Else If (Hit_Right = 1) {
			Return 0xE
		} Else {
			Return 0xC
		}
	} Else If (Hit_Bottom = 1) {
		If (Hit_Left = 1) {
			Return 0x10
		} Else If (Hit_Right = 1) {
			Return 0x11
		} Else {
			Return 0xF
		}
	} Else If (Hit_Left = 1) {
		Return 0xA
	} Else If (Hit_Right = 1) {
		Return 0xB
	}
	; Else let default hit-testing be done
}
; =======================================================================================